#include <cstdio>
#include <iostream>
#include <fstream>
#include "gluvi.h"
#include "vec3f.h"

using namespace std;

char frame_number[100]="frame 0";
const char *file_format;
unsigned int frame=0;
float radius;
vector<Vec3f> x;

bool read_frame(int newframe)
{
   if(newframe<0) return false;

   char filename[100];
   sprintf(filename, file_format, newframe);
   ifstream in(filename);
   if(!in.good())
      return false;

   in>>radius;
   float junk; in>>junk;
   int n;
   in>>n;
   x.resize(n);
   for(int i=0; i<n; ++i)
      in>>x[i];
   frame=newframe;

   sprintf(frame_number, "frame %d", frame);

   return true;
}

void set_view(Gluvi::Target3D &cam)
{
   cam.dist=3;
}

void set_lights_and_material(void)
{
   glEnable(GL_LIGHTING);
   {
      GLfloat ambient[4] = {.3, .3, .3, 1};
      glLightModelfv (GL_LIGHT_MODEL_AMBIENT,ambient);
   }
   {
      GLfloat color[4] = {.8, .8, .8, 1};
      glLightfv (GL_LIGHT0, GL_DIFFUSE, color);
      glLightfv (GL_LIGHT0, GL_SPECULAR, color);
      glEnable (GL_LIGHT0);
   }
   {
      GLfloat color[4] = {.4, .4, .4, 1};
      glLightfv (GL_LIGHT1, GL_DIFFUSE, color);
      glLightfv (GL_LIGHT1, GL_SPECULAR, color);
      glEnable (GL_LIGHT1);
   }
   {
      GLfloat color[4] = {.2, .2, .2, 1};
      glLightfv (GL_LIGHT2, GL_DIFFUSE, color);
      glLightfv (GL_LIGHT2, GL_SPECULAR, color);
      glEnable (GL_LIGHT2);
   }

   GLfloat ambient[4] = {.3, .3, .3};
   GLfloat diffuse[4] = {.6, .8, .2};
   GLfloat specular[4] = {.4, .2, .8};
   glMaterialfv (GL_FRONT, GL_AMBIENT, ambient);
   glMaterialfv (GL_FRONT, GL_DIFFUSE, diffuse);
   glMaterialfv (GL_FRONT, GL_SPECULAR, specular);
   glMaterialf (GL_FRONT, GL_SHININESS, 32);
}

void display(void)
{
   set_lights_and_material();
   glMatrixMode(GL_MODELVIEW);
   for(unsigned int i=0; i<x.size(); ++i){
      glPushMatrix();
      glTranslatef(x[i][0], x[i][1], x[i][2]);
      glutSolidSphere(radius, 6, 6);
      glPopMatrix();
   }
}

struct CameraButton : public Gluvi::Button {
   CameraButton(const char *t) : Gluvi::Button(t) {}
   void action()
   {
      ofstream camera_output("mud_camera.rib");
      Gluvi::camera->export_rib(camera_output);
   }
};

struct ScreenShotButton : public Gluvi::Button{
   const char *filename_format;
   ScreenShotButton(const char *label, const char *filename_format_) : Gluvi::Button(label), filename_format(filename_format_) {}
   void action()
   { Gluvi::ppm_screenshot(filename_format, frame); }
};

void special_key_handler(int key, int x, int y)
{
   switch(key){
      case GLUT_KEY_LEFT:
         if(read_frame(frame-1))
            glutPostRedisplay();
         break;
      case GLUT_KEY_RIGHT:
         if(read_frame(frame+1))
            glutPostRedisplay();
         break;
      default:
         ;
   }
}

int main(int argc, char **argv)
{
   Gluvi::init("mud_opengl", &argc, argv);

   if(argc!=2){
      cerr<<"Expecting one argument: format for animation filenames"<<endl;
      cerr<<" (e.g. splash%04d is the format for files splash0000, splash0001, ...)"<<endl;
      return 1;
   }

   file_format=argv[1];   

   if(!read_frame(0))
      return 1;

   glutSpecialFunc(special_key_handler);

   Gluvi::Target3D cam;
   set_view(cam);
   Gluvi::camera=&cam;

   Gluvi::userDisplayFunc=display;

   Gluvi::StaticText frametext(frame_number);
   Gluvi::root.list.push_back(&frametext);

   Gluvi::StaticText keytext("use left/right arrows for frame control");
   Gluvi::root.list.push_back(&keytext);

   CameraButton cambutton("Export camera to RIB");
   Gluvi::root.list.push_back(&cambutton);

   char ppmfileformat[strlen(file_format)+5];
   sprintf(ppmfileformat, "%s.ppm", file_format);
   ScreenShotButton screenshot("Screenshot", ppmfileformat);
   Gluvi::root.list.push_back(&screenshot);

   Gluvi::run();
   return 0;
}

