#ifndef VEC3F_H
#define VEC3F_H

#include <cassert>
#include <cmath>
#include <iostream>

// eventually templatize the float (after debugging) to allow double version
struct Vec3f
{
   float v[3];

   Vec3f(void)
   {}

   Vec3f(float value_for_all)
   { v[0]=v[1]=v[2]=value_for_all; }

   template<class S>
   Vec3f(const S source[3])
   { v[0]=(float)source[0]; v[1]=(float)source[1]; v[2]=(float)source[2]; }

   Vec3f(float v0, float v1, float v2)
   { v[0]=v0; v[1]=v1; v[2]=v2; }

   float &operator[](int index)
   {
      assert(0<=index && (unsigned int)index<3);
      return v[index];
   }

   const float &operator[](int index) const
   {
      assert(0<=index && (unsigned int)index<3);
      return v[index];
   }

   Vec3f operator+=(const Vec3f &w)
   { v[0]+=w.v[0]; v[1]+=w.v[1]; v[2]+=w.v[2]; return *this; }

   Vec3f operator-=(const Vec3f &w)
   { v[0]-=w.v[0]; v[1]-=w.v[1]; v[2]-=w.v[2]; return *this; }

   Vec3f operator*=(float scalar)
   { v[0]*=scalar; v[1]*=scalar; v[2]*=scalar; return *this; }

   Vec3f operator/=(float scalar)
   { v[0]/=scalar; v[1]/=scalar; v[2]/=scalar; return *this; }
};

static inline float mag2(const Vec3f &a)
{ return a.v[0]*a.v[0] + a.v[1]*a.v[1] + a.v[2]*a.v[2]; }

static inline float mag(const Vec3f &a)
{ return std::sqrt(mag2(a)); }

static inline Vec3f operator-(const Vec3f &a)
{ return Vec3f(-a.v[0], -a.v[1], -a.v[2]); }

static inline Vec3f operator+(const Vec3f &a, const Vec3f &b)
{ return Vec3f(a.v[0]+b.v[0], a.v[1]+b.v[1], a.v[2]+b.v[2]); }

static inline Vec3f operator-(const Vec3f &a, const Vec3f &b)
{ return Vec3f(a.v[0]-b.v[0], a.v[1]-b.v[1], a.v[2]-b.v[2]); }

static inline Vec3f operator*(const Vec3f &a, float scalar)
{ return Vec3f(scalar*a.v[0], scalar*a.v[1], scalar*a.v[2]); }

static inline Vec3f operator*(float scalar, const Vec3f &a)
{ return Vec3f(scalar*a.v[0], scalar*a.v[1], scalar*a.v[2]); }

static inline Vec3f operator/(const Vec3f &a, float scalar)
{ return Vec3f(a.v[0]/scalar, a.v[1]/scalar, a.v[2]/scalar); }

static inline float dot(const Vec3f &a, const Vec3f &b)
{ return a.v[0]*b.v[0] + a.v[1]*b.v[1] + a.v[2]*b.v[2]; }

static inline Vec3f cross(const Vec3f &a, const Vec3f &b)
{ return Vec3f(a.v[1]*b.v[2]-a.v[2]*b.v[1], a.v[2]*b.v[0]-a.v[0]*b.v[2], a.v[0]*b.v[1]-a.v[1]*b.v[0]); }

static inline void normalize(Vec3f &a)
{ a*=1./mag(a); }

static inline std::ostream &operator<<(std::ostream &out, const Vec3f &a)
{ return out<<a.v[0]<<' '<<a.v[1]<<' '<<a.v[2]; }

static inline std::istream &operator>>(std::istream &in, Vec3f &a)
{ return in>>a.v[0]>>a.v[1]>>a.v[2]; }

#endif
