#include "hashgrid.h"

unsigned int hash(const IntTriple &t)
{
   int h=(t.i<<5 + t.i) ^ t.j;
   h=(h<<5 + h) ^ t.k;
   return (unsigned int)(h<<5 + h) ^ 0xd3a795b3u;
}

HashGrid3D::finder HashGrid3D::find(const Vec3f &centre, float radius) const
{
   return finder(*this, centre, radius);
}

HashGrid3D_find_iterator::HashGrid3D_find_iterator (const HashGrid3D &hg_, const Vec3f &centre, float radius)
   : hg(hg_), lister(0)
{
   ilow=(int)((centre[0]-radius)*hg.overdx);
   ihigh=(int)((centre[0]+radius)*hg.overdx);
   jlow=(int)((centre[1]-radius)*hg.overdy);
   jhigh=(int)((centre[1]+radius)*hg.overdy);
   klow=(int)((centre[2]-radius)*hg.overdz);
   khigh=(int)((centre[2]+radius)*hg.overdz);
   cell.i=ilow;
   cell.j=jlow;
   cell.k=klow;
   while(!hg.grid.has_entry(cell)){
      ++cell.i;
      if(cell.i>ihigh){
         cell.i=ilow;
         ++cell.j;
         if(cell.j>jhigh){
            cell.j=jlow;
            ++cell.k;
            if(cell.k>khigh)
               return; // done - no points anywhere
         }
      }
   }
   lister=new HashTable<IntTriple, int>::list_iterator(hg.grid, cell);
}

void HashGrid3D_find_iterator::operator++()
{
   ++(*lister);
   if(!lister->still_going()){
      // get rid of this cell's iterator
      delete lister; lister=0;
      // move on until we find a cell with a point inside it
      do{
         ++cell.i;
         if(cell.i>ihigh){
            cell.i=ilow;
            ++cell.j;
            if(cell.j>jhigh){
               cell.j=jlow;
               ++cell.k;
               if(cell.k>khigh)
                  return; // we're done
            }
         }
      }while(!hg.grid.has_entry(cell));
      // and finally allocate new iterator
      lister=new HashTable<IntTriple, int>::list_iterator(hg.grid, cell);
   }
}

