#include <cstdarg>
#include <cstdio>
#include <cstdlib>
#include <fstream>
#include "mud.h"

using namespace std;

static float rand(float min, float max)
{ return (max-min)*(rand()/(float)RAND_MAX)+min; }

void Mud::init_sphere(const Vec3f &centre, float radius)
{
   int i, j, k;
   float grid_spacing=0.707106781186547524400844362*spring_restlength;
   int extent=(int)(radius/grid_spacing);

   for(i=-extent; i<=extent; ++i){
      for(j=-extent; j<=extent; ++j){
         for(k=-extent; k<=extent; ++k){
            if((i+j+k)%2==0){ // if this is part of the close-packed structure
               Vec3f offset=Vec3f(i+rand(-0.1,0.1), j+rand(-0.1,0.1), k+rand(-0.1,0.1))*grid_spacing;
               if(mag(offset)<=radius) // and if this is inside the sphere
                  x.push_back(centre+offset);
            }
         }
      }
   }
   n=x.size();
   v.resize(n);
   for(i=0; i<n; ++i)
      v[i]=Vec3f(0,0,0);
   f.resize(n);
   oldx.resize(n);
}

void Mud::advance_frame(float frame_dt)
{
   int steps=(int)ceil(frame_dt/max_dt);
   float dt=frame_dt/steps;
   std::cout<<"sub step size: "<<dt<<" (taking "<<steps<<" steps)"<<endl;
   for(int i=0; i<steps; ++i){
      cout<<i<<endl;
      advance_time_step(dt);
   }
   cout<<endl;
}

void Mud::write_output(char *filename_format, ...)
{
   va_list ap;
   va_start(ap, filename_format);
   char *filename;
   vasprintf(&filename, filename_format, ap);
   ofstream out(filename);
   free(filename);
   va_end(ap);

   out<<spring_restlength<<' '<<radius_of_influence<<endl; // possibly useful for rendering
   out<<n<<endl;
   for(int i=0; i<n; ++i)
      out<<x[i]<<endl;
}

void Mud::advance_time_step(float dt)
{
   // @@@ You need to fill in code here to do Symplectic Euler to advance each particle by time dt,
   // @@@ and to call the collision resolution routine collisions.resolve_inelastic_particle_collisions()
   // @@@ at the end.
}

void Mud::compute_forces()
{
   int i, j;

   // initialize f to just the force due to gravity on each particle
   for(i=0; i<n; ++i)
      f[i]=Vec3f(0, -gravity, 0); // technically this is wrong (should include a factor of mass) but this isn't a big issue for graphics

   // update the hashgrid to allow for fast searches for nearby particles
   grid.update();

   // loop over all particles
   for(i=0; i<n; ++i){
      // loop over all nearby particles (within distance of radius_of_influence)
      for(HashGrid3D::finder p(grid, x[i], radius_of_influence); p.still_going(); ++p){
         j=*p; // j is a nearby particle
         if(i==j || mag(x[i]-x[j])>radius_of_influence) continue; // don't do forces with i itself or ones too far away

	 // @@@ Here you need to add to f[i] the force due to particle j.
	 // @@@ This force is a damped spring of given restlength, multiplied by
	 // @@@ (1-dist/radius_of_influence) to smoothly decrease the force to zero
	 // @@@ for particles far away.
      }
   }
}

